/*
 *
 *	Header of IO operations for quota utilities
 *
 */

#ifndef _QUOTAIO_H
#define _QUOTAIO_H

#include <limits.h>
#include <sys/types.h>
#include <sys/stat.h>

#include "quota.h"

/* Latest known versions */
#define INITKNOWNVERSIONS {\
	0,\
	0\
}

#define QUOTAFORMATS 4

#define INITQFBASENAMES {\
	"quota",\
	"aquota",\
	"",\
	""\
}

#define INITQFMTNAMES {\
	"vfsold",\
	"vfsv0",\
	"rpc",\
	"xfs"\
}

/* Values for format handling */
#define QF_UNKNOWN -3		/* Format cannot be detected from filename */
#define QF_TOONEW -2		/* Quota format is too new to handle */
#define QF_ERROR -1		/* There was error while detecting format (maybe unknown format...) */
#define QF_VFSOLD 0		/* Old quota format */
#define QF_VFSV0 1		/* New quota format - version 0 */
#define QF_RPC 2		/* RPC should be used on given filesystem */
#define QF_XFS 3		/* XFS quota format */

/*
 * Definitions for disk quotas imposed on the average user
 * (big brother finally hits Linux).
 *
 * The following constants define the default amount of time given a user
 * before the soft limits are treated as hard limits (usually resulting
 * in an allocation failure). The timer is started when the user crosses
 * their soft limit, it is reset when they go below their soft limit.
 */
#define MAX_IQ_TIME  604800	/* (7*24*60*60) 1 week */
#define MAX_DQ_TIME  604800	/* (7*24*60*60) 1 week */

#endif /* _QUOTAIO_H */
